/**
 * JASMINe
 * Copyright (C) 2008 Bull S.A.S.
 * Contact: jasmine@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: Activator.java 7600 2011-02-01 10:06:27Z jlegrand $
 * --------------------------------------------------------------------------
 */
package org.ow2.jasmine.rules.cluster.cmi.osgi;

import org.osgi.framework.BundleContext;
import org.osgi.framework.InvalidSyntaxException;
import org.osgi.framework.ServiceEvent;
import org.osgi.framework.ServiceListener;
import org.osgi.framework.ServiceReference;
import org.ow2.cmi.controller.common.ClusterViewManager;
import org.ow2.cmi.controller.server.ServerClusterViewManager;
import org.ow2.jasmine.rules.cluster.cmi.CMIAction;
import org.ow2.jasmine.rules.osgi.impl.RulesActivator;

/**
 * @author Loris Bouzonnet
 */
public class Activator extends RulesActivator {

    private ServerClusterViewManager clusterViewManager;

    private ServiceReference clusterViewManagerReference;

    private ServiceListener clusterViewManagerListener;


    @Override
    public void doStart(final BundleContext bundleContext) throws Exception {
    	
    	// Name a specific target WorkingMemory (optional)
    	workingMemoryName = "cluster";
    	
    	// Specific CMI configuration
        clusterViewManagerListener = new ServiceListener() {
            public void serviceChanged(final ServiceEvent event) {
                ServiceReference ref = event.getServiceReference();
                // React on REGISTERED|UNREGISTERING events
                switch(event.getType()) {
                    case ServiceEvent.REGISTERED:
                        clusterViewManager = (ServerClusterViewManager) bundleContext.getService(ref);
                        CMIAction.getCMIAction().setClusterViewManager(clusterViewManager);
                        break;
                    case ServiceEvent.UNREGISTERING:
                        CMIAction.getCMIAction().setClusterViewManager(null);
                        clusterViewManager = null;
                        break;
                }
            }
        };
        try {
            bundleContext.addServiceListener(clusterViewManagerListener,
                    "(objectClass=" + ClusterViewManager.class.getName() + ")");
        } catch (InvalidSyntaxException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }
        clusterViewManagerReference = bundleContext.getServiceReference(
                ClusterViewManager.class.getName());
        if(clusterViewManagerReference != null) {
            clusterViewManager = (ServerClusterViewManager) bundleContext.getService(
                    clusterViewManagerReference);
            // Add CMIAction as global
            CMIAction cmiAction = CMIAction.getCMIAction();
            cmiAction.setClusterViewManager(clusterViewManager);
            setGlobal("cmiAction", cmiAction);
        }

    }

    @Override
    public void doStop(final BundleContext bundleContext) throws Exception {

        CMIAction.getCMIAction().destroy();
        if(clusterViewManagerListener != null) {
            bundleContext.removeServiceListener(clusterViewManagerListener);
            clusterViewManagerListener = null;
        }
        if(clusterViewManagerReference != null) {
            bundleContext.ungetService(clusterViewManagerReference);
            clusterViewManagerReference = null;
        }
        clusterViewManager = null;
    }

}
